package mmTest;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;

import org.json.JSONException;
import org.json.JSONObject;


import mmJava.MMConstants;
import mmJava.MMParam;
import mmJava.MMStats;
import mmRestAPI.MMAbstractRestAPI;
import mmRestAPI.MMAbstractRestAPI.FindCondition;
import mmRestAPI.MMAbstractRestAPI.RestVerb;
import mmRestAPI.MMRestAPI;
import mmRestAPI.MMRestApiCallback;
import mmRestAPI.MMRestApiUtil;
import mmServerCommunication.MMClient;
import mmServerCommunication.MMServerComm;
import mmServerCommunication.MMClient.EventType;
import mmServerCommunication.MMServerComm.ClientType;
import mmServerCommunication.MMServerComm.MMGetProjectListCallback;

/** 
 *	This Java API test class simply go through six basic operations in MassMobile.
 *	Simply creating a eclipse project, adding the external libs/jars and running this test
 *	as a Java application will start this test as a console program.
 *	Once it is started, typing the return key will get over each step (6 in total).
 *	What each step does can be viewed in the keyTyped(int c) function below.
 *	In short, they are:
 *	1) set an existing project as the current active project;
 *	2) start a new parameter list (use the commented getCurrentParamListName() if you want use an
 *	existing param list);
 *	3) print all the parameters of the current project;
 *	4) store/set a dummy parameter in server/Parse;  
 *	5) get parameters in the param list, data newer than the date string specified in there will be retrieved;
 *	6) get the statistics, i.e. average, median, etc. of the parameters in server/Parse.
 * 
 */

public class mmTestClient implements MMClient {
	static MMRestAPI restAPI = new MMRestAPI();
	static MMRestApiUtil apiUtil = new MMRestApiUtil();
	static MMAbstractRestAPI apiIns = restAPI.newInstance();
	static mmTestClient testClient = new mmTestClient();
	static MMStats stats = new MMStats();
	final static MMServerComm comm = new MMServerComm(restAPI, apiUtil, stats, ClientType.GENERAL_JAVA, testClient);
	
	public static void main(String [] args) {
		apiUtil.setServerComm(comm);
		//set the server URL
		testClient.setServerURL("http://massmobile.herokuapp.com");
		testClient.getProjectList(comm);
		String sWhatever;
		
	    Scanner scanIn = new Scanner(System.in);
	    for (int i = 0; i < 6; ++i) {
	    		sWhatever = scanIn.nextLine();
	    		testClient.keyTyped(i);
	    }
	 
	    scanIn.close();
	}

	int tempcounter = 0;
	
	public static void getProjectList(MMServerComm comm) {
		HashMap<Integer, String> projs;
		comm.getProjectList(comm.new MMGetProjectListCallback() {

			@Override
			public void done(HashMap<Integer, String> projectList) {
				for (Integer key : projectList.keySet())
					System.out.println(key.toString() + " " + projectList.get(key));	
			}
			
		});
		
	}
	
	public void keyTyped(int c) {
		
		if (c == 0)
			comm.setCurrentProject(1, true);
		else if(c == 1) {
			comm.startNewParamList();
			//use the getCurrentParamListName() to reuse the previously created Param List
			//comm.getCurrentParamListName();
		}
		else if(c == 2) {
			List<MMParam> params = comm.getParamsOfCurrentProject();
			for (int i = 0; i < params.size(); ++i) {
				System.out.println("PARAM_ID: " + params.get(i).getParamId() + "\t\tPARAM_NAME: " + params.get(i).getParamName());
				HashMap<String, Object> kvs = params.get(i).getkeyValues();
				for (String key : kvs.keySet()) {
					System.out.println("KEY: " + key + "\t\tVALUE: " + kvs.get(key));
				}
			}
		}
		else if (c == 3) {
			HashMap<String, Object> kv = new HashMap<String, Object>(); 
			kv.put("vote", 1);
			comm.addData("yourUserId", 1, 2, kv);
		}
		else if (c == 4) {
			//specify a time using an epoch millisecond time value as the 2nd parameter, this time needs to use the same time region as server time
			long time = 1385346589620L;
			comm.getParameters(1, time);
		}
		else if (c == 5) {
			comm.getStats(1, 2, 60);
		}
	}
	
	@Override
	public void changeListener(EventType event) {
		switch(event) {
		case PROJECT_LOADED:
			System.out.println("ProjectLoaded");
			break;
		case PARAMETER_ADDED:
			System.out.println("ParameterAdded");
			break;
		case PARAMETER_REMOVED:
			System.out.println("ParameterRemoved");
			break;
		case PARAMETER_SET:
			System.out.println("ParameterSet");
			break;
		case PARAMETERS_GOT:
			System.out.println("ParametersGot");
			break;
		default: break;
		}
	}

	@Override
	public void notifyError(String errorMessage) {
		System.out.println("error: " + errorMessage);
	}

	@Override
	public void notifyMessage(String string, String string2) {
		System.out.println("message: " + string + " + " + string2);
	}

	@Override
	public void getParameters(ArrayList<String> returnList, Object jsonArray_) {
		System.out.println(returnList.toString());
		for (int i=0; i < returnList.size(); i++)
			System.out.println("returnList[" + i + "]=" + returnList.get(i));
	}

	@Override
	public void returnedStats(ArrayList<String> parsedStats, Object jsonArray_) {
		for (int i=0; i < parsedStats.size(); i++)
			System.out.println("stats[" + i + "]=" + parsedStats.get(i));
	}

	@Override
	public void setServerURL(String url) {
		comm.setServerURL(url);
	}
}
