package mmMaxClient;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import mmJava.MMConstants;
import mmJava.MMParam;
import mmJava.MMStats;
import mmRestAPI.MMRestAPI;
import mmRestAPI.MMRestApiUtil;
import mmServerCommunication.MMClient;
import mmServerCommunication.MMServerComm;
import mmServerCommunication.MMServerComm.ClientType;

import com.cycling74.max.Atom;
import com.cycling74.max.MaxObject;

public class MMMaxClient extends MaxObject implements MMClient {
	MMServerComm serverComm = null;
	public static MMMaxClient instance = null;
	private MMRestAPI restAPI = null;
	private MMRestApiUtil apiUtil = null;
	private MMStats statsParser = null;
	public int lookback = MMConstants.DEFAULT_AGG_WINDOW;
	
	//Stores info about which messages to print
	private HashMap<String,Boolean> printHash = new HashMap<String,Boolean>();

	//Stores info about which messages to output
	private HashMap<String,Boolean> outputHash = new HashMap<String,Boolean>();
	
	public static MMMaxClient getInstance() {
		if (instance == null) {
			instance = new MMMaxClient();
			
		}
		return instance;
	}
	
	public MMMaxClient() {
		restAPI = new MMRestAPI();
		apiUtil = new MMRestApiUtil();
		statsParser = new MMStats();
		serverComm = new MMServerComm(restAPI, apiUtil, statsParser, ClientType.GENERAL_JAVA, this);
		//important for using servercomm!
		apiUtil.setServerComm(serverComm);
		instance = this;
		
		printHash.put("polling", true);
		printHash.put("statsRequest", true);
		printHash.put("guiControlMessages", true);
		printHash.put("postRawData", false);
		outputHash.put("guiControlMessages", true);
	}
	
	/** Creates a new project
	 * @param projectName
	 * 
	 */
	public void createNewProject(String projectName){
		serverComm.newProject(projectName);
	}
	
	/**
	 * Get the last ParamList table name on server and reuse it
	 */
	public void getCurrentParamListName()
	{
		serverComm.getCurrentParamListName();
	}
	
	/**
	 * Create a new param list table in the server dedicated to the current performance session
	 */
	public void startNewParamList()
	{
		serverComm.startNewParamList();
	}
	
	/**Allows users to choose which messages will show up in the max console 
	 * @param messageType
	 * @param toBool
	 */
	public void printToMax(String messageType, int toBool){
		boolean value;
		if (toBool==0){ value=false;}
		else{ value=true;}
		printHash.put(messageType, value);
	}

	private boolean printMessageType(String messageType) {
		return printHash.get(messageType);
	}

	public void outputToMax(String messageType, int toBool){
		boolean value;
		if (toBool==0){ value=false;}
		else{ value=true;}
		outputHash.put(messageType, value);
	}

	private boolean outputMessageType(String messageType){
		return outputHash.get(messageType);
	}
	
	/**
	 * Outputs a list of projectId and projectName
	 */
	public void getProjectList(){

		//HashMap<Integer, String> projectList;
		//projectList = serverComm.getProjectList();
		
		//TODO, change the weird java here later, i.e. serverComm.new , find an alternative
		serverComm.getProjectList(serverComm.new MMGetProjectListCallback() {

			@Override
			public void done(HashMap<Integer, String> projectList) {
				if(projectList != null){
					Iterator<Integer> it = projectList.keySet().iterator();
					while (it.hasNext()){
						//here: in database, the projIds need to be exactly as, 1,2,3,4,5,....
						int projectId = it.next();
						String projectName = projectList.get(projectId);
						outlet(0,"projectList " +projectId + " " +projectName);
					}
				} else{
					notifyError("Project List Getting Error");
				}
				
			}
			
		});
		
	}
	
	/**
	 * @param projectId
	 */
	public void setCurrentProject(int projectId){
		serverComm.setCurrentProject(projectId, true);
	}
	
	/**
	 * Gets the project info for the current project.
	 */
	public void getProjectInfo() {
		outlet(0,"projectInfo " +serverComm.getProjectInfo());
	}
	
	/**Sets the project info for the current project.
	 * @param info
	 */
	public void setProjectInfo(String info){
		serverComm.setProjectInfo(info);
	}
	
	/**
	 * Adds Param to project and calls getParameterList() to update Max
	 * ex. paramType (SLIDER) 
	 * @param paramName
	 * @param paramType
	 */
	public void addParamToCurrentProject(String paramName, int paramType){
		serverComm.addParameter(paramName, paramType);
		getParameterList();//output parameter list with new parameter
	}


	/**Removes Param from the current project and calls getParameterList() to update Max
	 * @param paramId
	 */
	public void removeParameter(int paramId){
		serverComm.removeParameter(paramId);
		getParameterList();
	}

	/**
	 * Sets the key value pair for the Param
	 * This does not send to parse until updateProject() is called
	 * @param paramName
	 * @param key
	 * @param value
	 */
	public void setParamConfigs(Atom[] args){
		Object value = null;
		int paramId = -1;
		if(args.length!=3){
			post("setParamConfigs: Invalid number of arguments");
		} else{

			paramId = args[0].toInt();
			String key = args[1].toString();
//			if (key.equals("countdownStart")) {
//				String temp = (String)args[2].toObject();
//
//				DateFormat m_ISO8601Local = new SimpleDateFormat ("yyyy-MM-dd'T'HH:mm:ss.SSS");
//				Date tempDate = null;
//				try {
//					tempDate = m_ISO8601Local.parse(temp);
//				} catch (ParseException e) {
//					System.out.println("iso string parse error");
//					e.printStackTrace();
//				}
//				value = tempDate.getTime();
//				System.out.println(tempDate.toString());
//				System.out.println(value.toString());
//			}
//			else {
//				value = args[2].toObject();
//			}
			value = args[2].toObject();

			//post(key);
			//post(value.toString());
			MMParam thisParam = serverComm.getParameterConfig(paramId) ;
			thisParam.setKeyValue(key, value);
			getParameterConfigs(paramId);
		}	
	}
	
	private Object printType() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Sends any paramValues that have changed to Parse
	 */
	public void updateProject(){
		serverComm.updateProject();
	}
	
	/**Refreshes changes to the parameterList and configsColl
	 * This method can not be called from Max.
	 */
	private void getParameterList(){
		if (outputMessageType("guiControlMessages")){
			outlet(0, "parameterList clear");
			outlet(0, "configsColl clear");
		}
		List<MMParam> parameters = serverComm.getParamsOfCurrentProject();
		if(parameters!=null){
			for (int i = 0; i<parameters.size();i++){
				MMParam aParam = parameters.get(i);
				outlet(0, "parameterList " + aParam.getParamId() + " " + aParam.getParamType() + " " + aParam.getParamName());
			}
		}
	}
	
	/**Gets the configuration of the given parameter of the current project and outputs to Max.
	 * @param paramId
	 */
	public void getParameterConfigs(int paramId){
		if (outputMessageType("guiControlMessages")){
			outlet(0, "configsColl clear");
		}

		List<MMParam> parameters = serverComm.getParamsOfCurrentProject();
		if(parameters!=null){
			for (int i = 0; i<parameters.size();i++){
				MMParam thisParam = parameters.get(i);
				if (thisParam.getParamId()==paramId){

					HashMap<String, Object> keyValues = thisParam.getkeyValues();
					outlet(0,"configsColl paramType " + thisParam.getParamType());
					Iterator<String> it = keyValues.keySet().iterator();
					String message = null;
					String key;
					int counter = 0; //used to easily add to coll
					while (it.hasNext()){
						counter++;
						key = it.next();
						message= " " + counter + " " + key + " " + keyValues.get(key);
						outlet(0,"configsColl " + message);
					}
					outlet(0, "paramId " +paramId);
					
					post(keyValues.toString());
				}
			}
		}
	}
	
	/**Sets the poll rate in milliseconds.
	 * @param pollRate
	 */
	public void setPollRate(int pollRate){
		serverComm.setPollRate(pollRate);
	}

	/**Starts and stops polling the statistics for the given paramId. 
	 * pollSwitch==0 stops polling stats.
	 * pollSwitch==1 starts polling stats.
	 * @param paramId
	 * @param pollSwitch
	 */
	public void pollStats(int paramId, int pollSwitch){
		if (serverComm.getProject().getProjectName().equals("GTBand"))
			serverComm.pollStatsScaled(paramId,pollSwitch, lookback);
		else
			serverComm.pollStats(paramId,pollSwitch, lookback);
	}
	
	/**Starts and stops polling the raw data for the given paramId. 
	 * pollSwitch==0 stops polling raw data.
	 * pollSwitch==1 starts polling raw data.
	 * @param paramId
	 * @param pollSwitch
	 */
	public void pollRaw(int paramId, int pollSwitch){
		serverComm.pollRaw(paramId, pollSwitch, lookback);
	}
	
	public void setPollWindow(int seconds) {
		lookback = seconds;
	}


	public void postData(Atom[] args){

		String userId =args[0].toString();
		int paramId = args[1].toInt();
		int paramType = args[2].toInt();
		HashMap<String,Object> h = new HashMap<String,Object>();

		ArrayList<String> keys = MMConstants.getKeysForParamType(paramType);

		for (int i = 0; i < keys.size(); i++){
			if(args.length>(i+3)){
				h.put(keys.get(i), args[i+3].toFloat());
			} else {
				notifyError("invalid number of arguments");
				return;
			}
		}
		serverComm.addData(userId, paramId, paramType,h);
	}


	/**
	 * Stops all polling
	 */
	public void stopPolling(){
		serverComm.stopPolling();
	}


	public void getStats(int paramId, int paramType){
		serverComm.getStats(paramId, paramType, lookback);
	}

	@Override
	public void changeListener(EventType event) {
		switch(event) {
			case PROJECT_LOADED:
				//output projectId for the loaded project
				outlet(0,"projectId " + serverComm.getProjectId());
				getParameterList();
				getProjectInfo();
				break;
			case PROJECT_ADDED:
				getProjectList();
				break;
			case PARAMETER_SET:
				break;
			default: break;
		}
	}

	//gets raw data from ParamList table. Rename to something like getRawData
	@Override
	public void getParameters(ArrayList<String> newParams, Object jsonArray_) {
		for (int i = 0 ; i < newParams.size();i++){
			String message = "rawData " +newParams.get(i);
			outlet(0,message);
			notifyMessage("postRawData",message);
		}
	}

	@Override
	public void returnedStats(ArrayList<String> returnedStats, Object jsonArray_) {
		Iterator<String> it = returnedStats.iterator();
		while(it.hasNext()){
			String next = it.next();
			outlet(0,"stats " + next);
		}		
	}

	@Override
	public void notifyError(String errorMessage) {
		error(errorMessage);		
	}

	@Override
	public void notifyMessage(String messageType, String message) {
		if(printMessageType(messageType)){
			post(message);
		}		
	}

	@Override
	public void setServerURL(String url) {
		serverComm.setServerURL(url);
		
	}
	
	public void resetStatsTimer() {
		serverComm.resetStatsTimer();
	}
}
