/**
  ==============================================================================
  web.js
  MassMobile-nodeRest-javaAPI

  Created by Shaoduo Xie on Oct 2013
  Copyright (c) 2012~2013 GTRC. All rights reserved.
  ==============================================================================
 */


// sub for __dirname, used at the end of mydb.openDatabase()
process.env.PWD = process.cwd()

var mydb = require('./mydb');
var mongodb= require('mongodb');
var express = require('express');
var BSON = mongodb.BSONPure;
var app = express();


// open the database once
mydb.openDatabase(function(err, db) {
  if (err) {
    console.log('ERROR CONNECTING TO DATABASE');
    console.log(err);
    process.exit(1);
  }

  mydb.authenticate(db, 'mmteam', 'teammm', function(err, coll) {
    if (err) {
      console.log('ERROR AUTHENTICATING');
      console.log(err);
      process.exit(1);
    }

    app.post('/add/:collection', function(req, res) {
      var body = '';
      req.on('data', function (data) {
        body += data;
        // console.log("body " + body);
      });
      req.on('end', function () {
        
        var obj = null;
        if (body) {
          obj = JSON.parse(body);
          // console.log('Adding object: ' + JSON.stringify(obj));
        }
        else {
          console.log("error: no body");
          res.send("error: no body");
          return;
        }
        
        var collectionName = req.params.collection;
        var collection = db.collection(collectionName);
        var toAdd = obj;
        // console.log('toadd: ' + toAdd);
        if (!toAdd) {
          console.log('nothing to add');
          return;
        }
        //toAdd['updatedAt'] = new Date().toISOString();
        toAdd['updatedAt'] = new Date().getTime();
        toAdd['createdAt'] = new Date().getTime();
        
        collection.insert(toAdd, {safe:true}, function(err, result) {
            if (err) {
                res.send({'error':'An error has occurred'});
            } else {
                // console.log('Success: ' + JSON.stringify(result[0]));
                var toRet = {};
                toRet['results'] = {'retArray' : result};
                // console.log(toRet);
                res.send(toRet);
            }
        });

      });
    });

    app.post('/find/:collection', function(req, res) {
      var body = '';
      req.on('data', function (data) {
        body += data;
        //console.log("body " + body);
      });
      req.on('end', function () {

        var obj = null;
        var constraint = null;
        var getCount = false;
        var limit = -1;
        if (body) {
          obj = JSON.parse(body);
          //console.log('constraint ' + JSON.stringify(obj));
          constraint = obj.q;
          getCount = (obj.c) ? obj.c : false;
          limit = (obj.limit) ? obj.limit : -1;
        }
        else {
          constraint = {};
        }
        var collectionName = req.params.collection;
        var collection = db.collection(collectionName);
        
        //console.log('constraint: ' + constraint);

        if (limit == -1) {
          collection.find(constraint).toArray(function(err, items) {
              if (err) {
                  console.log('Error updating: ' + err);
                  res.send({'error':'An error has occurred'});
              } else {
                  var toRet = {};
                  toRet['results'] = {'retArray' : items};
                  if (getCount)
                    toRet['count'] = items.length;
                  //console.log('returning' + items);
                  res.send(toRet);
              }
          });
        }
        else {
          //with limit
          collection.find(constraint).limit(limit).toArray(function(err, items) {
              if (err) {
                  console.log('Error updating: ' + err);
                  res.send({'error':'An error has occurred'});
              } else {
                  var toRet = {};
                  toRet['results'] = {'retArray' : items};
                  if (getCount)
                    toRet['count'] = items.length;
                  //console.log(items);
                  res.send(toRet);
              }
          });
        }
      });
    });

    app.put('/update/:collection/:id', function(req, res) {
      var body = '';
      req.on('data', function (data) {
        body += data;
        console.log("body " + body);
      });
      req.on('end', function () {
        //var POST = qs.parse(body);
        var id = req.params.id;
        var obj = null;
        if (body && id) {
          obj = JSON.parse(body);
          // use POST
          console.log('Adding object: ' + JSON.stringify(obj));
        }
        else {
          console.log('error: no body or id');
          res.send('error: no body or id');
          return;
        }

        var collectionName = req.params.collection;
        var collection = db.collection(collectionName);
        
        var toAdd = obj;
        console.log('toadd: ' + toAdd);
        if (!toAdd) {
          console.log('nothing to add');
          return;
        }

        collection.update({'_id':new BSON.ObjectID(id)}, toAdd, {safe:true}, function(err, result) {
            if (err) {
                console.log('Error updating: ' + err);
                res.send({'error':'An error has occurred'});
            } else {
                console.log('' + result + ' document(s) updated');
                //res.send(obj);
                var toRet = {};
                var dummyArray = new Array();
                dummyArray[0] = {'retVal':result};
                toRet['results'] = {'retArray' : dummyArray};
                res.send(toRet);
            }
        });

      });

    });

    app.delete('/delete/:collection/:id', function(req, res) {
      var collectionName = req.params.collection;
      var collection = db.collection(collectionName);

      var id = req.params.id;
      if (!id) {
        console.log("no id found");
        res.send("error: no id");
        return;
      }
      console.log('Deleting: ' + id);
      collection.remove({'_id':new BSON.ObjectID(id)}, {safe:true}, function(err, result) {
          if (err) {
              res.send({'error':'An error has occurred - ' + err});
          } else {
              console.log('' + result + ' document(s) deleted');
              //res.send(req.body);
              var toRet = {};
              var dummyArray = new Array();
              dummyArray[0] = {'retVal':result};
              toRet['results'] = {'retArray' : dummyArray};
              res.send(toRet);
          }
      });
    });

    app.get('/getServerTime', function(req, res) {
      var result = new Date().getTime();
      var toRet = {};
      var dummyArray = new Array();
      dummyArray[0] = {'retVal':String(result)};
      toRet['results'] = {'retArray' : dummyArray};
      res.send(toRet);
    });


    app.post('/getStats', function(req, res) {
      var body = '';
      req.on('data', function (data) {
          body += data;

      });
      req.on('end', function () {
        var params = null;
        if (body) {
            params = JSON.parse(body);
            //console.log('params ' + JSON.stringify(params));
        }
        else {
            var emptyRet = {};
            res.send(emptyRet);
        }

        statsParams = JSON.parse(JSON.stringify(params));
        var interval = 0;
        if (!statsParams.interval) {
          interval = 200; //ms
        }

        var collection = db.collection('StatsTimer');
        collection.find().toArray(function(err, items) {
            if (err) {
              console.log('Error updating: ' + err);
              res.send({'error':'An error has occurred in finding StatsTimer'});
            } 
            else {
              if (items.length == 1) {
                if (items[0]['timerOn'] == 0) {
                  //start Timer
                  theDB = db;
                  statsTimer = setInterval(calStatsTimerCallback, 200);
                  timerObjId = items[0]['_id'];
                  var tuple = {'timerOn' : 1};
                  console.log('timerobjid: ' + timerObjId);
                  collection.update({'_id':new BSON.ObjectID(String(timerObjId))}, tuple, {safe:true}, function(err, result) {
                      if (err) {
                          console.log('Error updating: ' + err);
                          res.send({'error':'An error has occurred in getstats'});
                      } else {
                          console.log('' + result + ' document(s) updated');
                      }
                  });

                  res.send('no data');
                }
                else {
                  //timer in running, return last calculated data
                   //with limit
                  collection = db.collection('StatsRecord');
                  // collection.find().sort({updatedAt : -1}).limit(1).toArray(function(err, items) {
                  //     if (err) {
                  //       console.log('Error finding: ' + err);
                  //       res.send({'error':'An error has occurred in getstats'});
                  //     } 
                  //     else {
                  //       if (items.length == 1) {
                  //         res.send(items[0]);
                  //       }
                  //       else {
                  //         res.send('no data');
                  //       }                        
                  //     }

                  // });
                  collection.find().toArray(function(err, items) {
                      if (err) {
                        console.log('Error finding: ' + err);
                        res.send({'error':'An error has occurred in getstats'});
                      } 
                      else {
                        if (items.length == 1) {
                          res.send(items[0]);
                        }
                        else {
                          res.send('no data');
                        }                        
                      }

                  });

                }
              }

            }
        });
      });
    });

    app.get('/endEvent', function(req, res) {
      var collection = db.collection('StatsTimer');
      var tuple = {'timerOn' : 0};
      console.log('in end evnet');
      clearInterval(statsTimer);
      collection.find().toArray(function(err, items) {
        if (err) {
          console.log('Error updating: ' + err);
          res.send({'error':'An error has occurred in finding endEvent'});
        } 
        else {
          collection.update({'_id':new BSON.ObjectID(String(items[0]['_id']))}, tuple, {safe:true}, function(err, result) {
              if (err) {
                  console.log('Error updating: ' + err);
                  res.send({'error':'An error has occurred in endEvent'});
              } else {
                  console.log('' + result + ' document(s) updated');
                  res.send("event ended");
              }
          });

          // var statsReord = db.collection('StatsRecord');
          // statsReord.remove({},function(err, removed){
          // });
        }
      });
      
      //theDB = null;
      statsParams = {};
      statsTimer = null;
      timerObjId = null;
    });


    // app.post('/calStats', function(req, res) {
      // var body = '';
      // req.on('data', function (data) {
      //     body += data;
      //     //console.log("body " + body);
      // });
      // req.on('end', function () {
      //     //console.log('req end');

      //   var params = null;
      //   if (body) {
      //       params = JSON.parse(body);
      //       //console.log('params ' + JSON.stringify(params));
      //   }
      //   else {
      //       var emptyRet = {};
      //       res.send(emptyRet);
      //   }

    //     var curParamListCollection = db.collection("CurParamList");
    //     curParamListCollection.find().toArray(function(err, items) {
    //         if (err) {
    //             console.log('Error finding: ' + err);
    //             res.send({'error':'An error has occurred'});
    //         } 
    //         else {
    //           if (items.length != 1) {
    //             res.send("query for Current Param List should return only 1 object.");
    //             return;
    //           }
    //           var paramListName = items[0]['tableName'];

    //           //setting constraints
    //           var query = {};
    //           query['paramId'] = params.paramId;
    //           query['projId'] = params.projId;
    //           var timeEnd = new Date(params.timeOfRequest);
    //           var timeStart = new Date(params.timeOfRequest);
    //           var lookback = (params.window != null && params.window != undefined) ? params.window : 30;
    //           timeStart.setTime(timeStart.getTime() - lookback * 1000);
    //           //console.log(params.timeOfRequest);
    //           //console.log('iso: ' + timeStart);
    //           //console.log('iso: ' + timeStart.getTime());
    //           //query['updatedAt'] = {'$gte': timeStart.toISOString(), '$lte': timeEnd.toISOString()};
    //           query['updatedAt'] = {'$gte': timeStart.getTime(), '$lte': timeEnd.getTime()};
    //           var sentence = {};
    //           sentence['$query'] = query;
    //           sentence['$orderby'] = {'updatedAt' : -1};

    //           // console.log(sentence);
    //           // console.log(paramListName);

    //           var collection = db.collection(paramListName);
    //           collection.find(sentence).limit(0).toArray(function(err, data) {
    //               if (err) {
    //                   console.log('Error in dining: ' + err);
    //                   res.send({'error':'An error has occurred'});
    //               } 
    //               else {
    //                   if (data.length == 0) {
    //                     res.send("no data found, data.length == 0");
    //                     return;
    //                   } 

    //                   var ret = {};

    //                   var csliderId = 6;
    //                   if (params.paramType == csliderId)                 
    //                     ret['csliderAggr'] = calCappedAggregate(data, params);
    //                   else
    //                     ret = calStatistics(data, params);

    //                   if (!ret) {
    //                     res.send("ret of calStatisticsOnKey(..) is null");
    //                   }
    //                   else if (ret) {
    //                     ret['paramId'] = params.paramId;
                        
    //                     var toRet = {};
    //                     toRet['result'] = ret;
    //                     //console.log(toRet);
    //                     res.send(toRet);
    //                   }
    //               }
    //           });
    //         }
    //     });


       
    //   });

    // });

    // create a static subdir
    app.configure(function() {
      app.use(express.methodOverride());
      app.use(express.bodyParser());
      app.use('/public', express.static(process.env.PWD + '/public'));
      app.use(express.errorHandler({
        dumpExceptions: true, 
        showStack: true
      }));
      app.use(app.router);
    });

    app.get("/", function(req, res) {
      res.redirect("/public/MassMobile_web_client.html");
    });

    var port = process.env.PORT || 5000;
    app.listen(port, function() {
      console.log("Listening on " + port);
    });

    //gtband specific, scaled average
    var SCALE_NUM_HISTORY = 250;
    var SCALE_historyAvgX = new Array(SCALE_NUM_HISTORY);
    var SCALE_historyAvgY = new Array(SCALE_NUM_HISTORY);
    var SCALE_historyCursor = 0;
    var SCALE_maxAvgX = -10000;
    var SCALE_minAvgX = 10000;
    var SCALE_maxAvgY = -10000;
    var SCALE_minAvgY = 10000;
    app.post('/calStatsScaled', function(req, res) {
      var body = '';
      req.on('data', function (data) {
          body += data;
          //console.log("body " + body);
      });
      req.on('end', function () {
          //console.log('req end');

        var params = null;
        if (body) {
            params = JSON.parse(body);
            //console.log('params ' + JSON.stringify(params));
        }
        else {
            var emptyRet = {};
            res.send(emptyRet);
        }

        var curParamListCollection = db.collection("CurParamList");
        curParamListCollection.find().toArray(function(err, items) {
            if (err) {
                console.log('Error finding: ' + err);
                res.send({'error':'An error has occurred'});
            } 
            else {
              if (items.length != 1) {
                res.send("query for Current Param List should return only 1 object.");
                return;
              }
              var paramListName = items[0]['tableName'];

              //setting constraints
              var query = {};
              query['paramId'] = params.paramId;
              query['projId'] = params.projId;
              var timeEnd = new Date(params.timeOfRequest);
              var timeStart = new Date(params.timeOfRequest);
              var lookback = (params.window != null && params.window != undefined) ? params.window : 30;
              timeStart.setTime(timeStart.getTime() - lookback * 1000);
              //console.log(params.timeOfRequest);
              //console.log('iso: ' + timeStart);
              //console.log('iso: ' + timeStart.getTime());
              //query['updatedAt'] = {'$gte': timeStart.toISOString(), '$lte': timeEnd.toISOString()};
              query['updatedAt'] = {'$gte': timeStart.getTime(), '$lte': timeEnd.getTime()};
              var sentence = {};
              sentence['$query'] = query;
              sentence['$orderby'] = {'updatedAt' : -1};

              // console.log(sentence);
              // console.log(paramListName);

              var collection = db.collection(paramListName);
              collection.find(sentence).limit(0).toArray(function(err, data) {
                  if (err) {
                      console.log('Error in dining: ' + err);
                      res.send({'error':'An error has occurred'});
                  } 
                  else {
                      if (data.length == 0) {
                        res.send("no data found, data.length == 0");
                        return;
                      } 
                      var ret = calStatistics(data, params);
                      if (!ret) {
                        res.send("ret of calStatisticsOnKey(..) is null");
                      }
                      else if (ret) {
                        //scale
                        var SCALE_x = ret['normalUser']['x']['average'];
                        var SCALE_y = ret['normalUser']['y']['average'];
                        SCALE_historyAvgX[SCALE_historyCursor] = ret['normalUser']['x']['average'];
                        SCALE_historyAvgY[SCALE_historyCursor] = ret['normalUser']['y']['average'];
                        SCALE_maxAvgX = findMax(SCALE_historyAvgX);
                        SCALE_minAvgX = findMin(SCALE_historyAvgX);
                        SCALE_maxAvgY = findMax(SCALE_historyAvgY);
                        SCALE_minAvgY = findMin(SCALE_historyAvgY);
                        ++SCALE_historyCursor;
                        if (SCALE_historyCursor >= SCALE_NUM_HISTORY) {
                          SCALE_historyCursor = 0;
                        }

                        var rangeX = SCALE_maxAvgX - SCALE_minAvgX;
                        var rangeY = SCALE_maxAvgY - SCALE_minAvgY;
                        if (rangeX == 0) {
                          //nothing
                        }
                        else {
                          ret['normalUser']['x']['average'] = (ret['normalUser']['x']['average'] - SCALE_minAvgX) / rangeX;
                        }
                        if (rangeY == 0) {
                          //nothing
                        }
                        else {
                          ret['normalUser']['y']['average'] = (ret['normalUser']['y']['average'] - SCALE_minAvgY) / rangeY;
                        }

                        if (ret['normalUser']['x']['N'] == 1) {
                          ret['normalUser']['x']['average'] = SCALE_x;
                          ret['normalUser']['y']['average'] = SCALE_y;
                        }


                        ret['paramId'] = params.paramId;
                        
                        var toRet = {};
                        toRet['result'] = ret;
                        //console.log(toRet);
                        res.send(toRet);
                      }
                  }
              });
            }
        });


       
      });

    });
  });
});

//global
var statsTimer = null;
var timerObjId = null;
var statsParams = {};
var theDB = null;

function calStatsTimerCallback() {
  // console.log('!!!!!!!!!!!!!!caling stats!!!!!!!!!!!!!!!!!');
  if (theDB != null) {
    calStats(theDB, function(stats) {
      var collection = theDB.collection('StatsRecord');
      if (stats != null) {
        // console.log('@@@@@stats not null@@@@@');
        //stats['updatedAt'] = new Date().getTime();
        // collection.insert(stats, {safe:true}, function(err, result) {
        //     if (err) {
        //         res.send({'error':'An error has occurred'});
        //     } else {
        //       console.log('stats stored');
        //     }
        // });

        collection.update({'_id':new BSON.ObjectID(String("5348a14869e275020061a273"))}, stats, {safe:true}, function(err, result) {
            if (err) {
                console.log('Error updating: ' + err);
                res.send({'error':'An error has occurred in endEvent'});
            } else {
               console.log('new stats updated');
            }
        });
      }
      else {
        console.log('^^^stats is null');
      }
    }); 
  }
}

function calCappedAggregate(data, params)
{
  /* color-slider-specific aggregate function */
  var ret = {};
  if (data.length == 0) 
    return ret;

  var sliderAggrArray = {};
  for (var i = 0; i < 4; ++i)
  {
    sliderAggrArray[i] = 0.5;
  }

  for (var i = 0; i < data.length; ++i)
  {
    var combined = data[i]['keyValues']['cslider'];
    var id = Math.floor(combined / 2);
    var choice = combined % 2;
    var inc = data[i]['keyValues']['cslider_inc'] * (choice * 2 - 1);
    sliderAggrArray[id] += inc;

    if (sliderAggrArray[id] > 1) sliderAggrArray[id] = 1.0;
    if (sliderAggrArray[id] < 0) sliderAggrArray[id] = 0.0;
  }
  return sliderAggrArray;
}

function calStats(db, callback) {

  if (statsParams == null) {
    callback(null);
    return;
  }

  var curParamListCollection = db.collection("CurParamList");
  curParamListCollection.find().toArray(function(err, items) {
      if (err) {
          console.log('Error finding: ' + err);
      } 
      else {
        if (items.length != 1) {
          callback(null);
          return;
        }
        var paramListName = items[0]['tableName'];

        //setting constraints
        var query = {};
        query['paramId'] = statsParams.paramId;
        query['projId'] = statsParams.projId;
        var timeEnd = new Date();
        var timeStart = new Date(timeEnd.getTime());
        var lookback = (statsParams.window != null && statsParams.window != undefined) ? statsParams.window : 30;
        timeStart.setTime(timeStart.getTime() - lookback * 1000);
        //console.log(params.timeOfRequest);
        //console.log('iso: ' + timeStart);
        //console.log('iso: ' + timeStart.getTime());
        //query['updatedAt'] = {'$gte': timeStart.toISOString(), '$lte': timeEnd.toISOString()};
        query['updatedAt'] = {'$gte': timeStart.getTime(), '$lte': timeEnd.getTime()};
        var sentence = {};
        sentence['$query'] = query;
        sentence['$orderby'] = {'updatedAt' : -1};

        // console.log('$$$$startTime: ' + timeStart.getTime() + ' endTime: ' + timeEnd.getTime() + '$$$$\n');
        // console.log('paramlistname: ' + paramListName);
        // console.log('param id: ' + statsParams.paramId);
        // console.log('projid id: ' + statsParams.projId);

        var collection = db.collection(paramListName);
        collection.find(sentence).limit(0).toArray(function(err, data) {
            if (err) {
                console.log('Error in finding: ' + err);
                
            } 
            else {
                if (data.length == 0) {
                  console.log('data is zero');
                  callback(null);
                  return;
                } 

                var ret = {};

                var csliderId = 6;
                if (statsParams.paramType == csliderId) {                
                  ret['csliderAggr'] = calCappedAggregate(data, statsParams);
                }
                else
                  ret = calStatistics(data, statsParams);

                if (!ret) {
                  // res.send("ret of calStatisticsOnKey(..) is null");
                  console.log("ret is null");
                  callback(null);
                  return;
                }
                else if (ret) {
                  ret['paramId'] = statsParams.paramId;
                  
                  var toRet = {};
                  toRet['result'] = ret;
                  //console.log(toRet);
                  //res.send(toRet);
                  callback(toRet);
                  return;
                }
            }
          });
      }
    });

}

function calStatistics(data, params)
{
  var ret = {};
  if (data.length == 0) //actually this will never be true
    return ret;
  
  var duRecs = getDiffUsersRecs(data);  //different users records
  var suRecs = getSpecialUsersRecs(data, params.specialUsers);
  
  var keys = params.keys;
  var duResults = {};
  var suResults = {};
  for (var i = 0; i < keys.length; ++i) {
    var calRet = calStatisticsOnKey(duRecs, keys[i]);
    if (calRet) {
      duResults[keys[i]] = calRet;
    }

    if (suRecs.length > 0) {
      
      calRet = calStatisticsOnKey(suRecs, keys[i]);
      if (calRet) {
        suResults[keys[i]] = calRet;
      }
    }
  }
  ret['normalUser'] = duResults;
  ret['specialUser'] = suResults;
  
  //get raw data for special users
  var suRawData = {};
  for (var i = 0; i < suRecs.length; ++i) {
    var eachSU = {};
    for (var k = 0; k < keys.length; ++k) {
      //eachSU[keys[k]] = suRecs[i].get("keyValues")[keys[k]];
      eachSU[keys[k]] = suRecs[i]['keyValues'][keys[k]];
    }
    // suRawData[suRecs[i].get("userId")] = eachSU;
    suRawData[suRecs[i]['userId']] = eachSU;
  }
//  if (suRecs.length == 0) {
//    var fake = {};
//    fake['x'] = 0.3;
//    fake['y'] = 0.3;
//    suRawData['foo'] = fake; 
//    //suRawData['ret'] = request.params.specialUsers;
//  }
  ret['su_rawdata'] = suRawData;
  
  return ret;
}

function calStatisticsOnKey(data, key)
{
  var ret = {};
  var samples = new Array();
  var sum = 0;
  
  for (var i = 0; i < data.length; ++i) {
    //var value = data[i].get("keyValues")[key];
    var value = data[i]['keyValues'][key];
    if (value != null && value != undefined) {
      samples.push(value);
      sum += value;
    }
  }
  
  var N = data.length;
  if(N == 0 || samples.length == 0)
    return null;
  
  samples.sort(function(a, b) { return a - b });
  var average = sum / N;
  var sumOfDiffSquared = 0;
  var sumOfDiffCubed = 0;
  var sumOfDiffQuadded = 0;
  for (var i = 0; i < samples.length; ++i) {
    sumOfDiffSquared += (samples[i] - average) * (samples[i] - average);
    sumOfDiffCubed += Math.pow((samples[i] - average), 3);
    sumOfDiffQuadded += Math.pow((samples[i] - average), 4);
  }
  var sd = (N - 1) ? Math.sqrt(sumOfDiffSquared / (N - 1)) : -1;
  var variance = sumOfDiffSquared / N;
  var skewness = (N - 1) ? (sumOfDiffCubed / N) / Math.pow((sumOfDiffSquared / N), 1.5) : 0;
  var kurtosis = (N - 1) ? (sumOfDiffQuadded / N) / Math.pow((sumOfDiffSquared / N), 2) - 3 : -1;
  var median;
  if (N >= 1) {
    median = (N % 2) ? samples[(N + 1) / 2] : (samples[N / 2] + samples[(N - 2) / 2]) / 2;
  }
  else
    median = -10000;  //give it a bad number to indicate N < 1
  var min = samples[0];
  var max = samples[samples.length - 1];

  ret['N'] = N;
  ret['average'] = average;
  ret['sd'] = sd;
  ret['variance'] = variance;
  ret['min'] = min;
  ret['max'] = max;
  ret['median'] = median;
  ret['skewness'] = skewness;
  ret['kurtosis'] = kurtosis;
  ret['sumOfDiffSquared'] = sumOfDiffSquared;
  ret['sumOfDiffCubed'] = sumOfDiffSquared;
  ret['sumOfDiffQuadded'] = sumOfDiffQuadded;
  
  //vote param will do the following
  
  //var choices = new Array();
  var choices = {};
  
  for (var p = 0; p < data.length; ++p) {
    //if (p == 0 && data[p].get("paramType") != 2)
    if (p == 0 && data[p]['paramType'] != 2)
      //ugly check to see if this param is not a vote.., paramType == 2 means voting param
      break;
    //var value = data[p].get("keyValues")[key];
    var value = data[p]['keyValues'][key];
    for (var mv = 0; mv < value.length; ++mv) {
      if (value[mv] in choices)
        choices[value[mv]]++;
      else
        choices[value[mv]] = 1;
    }
//    if (value in choices)
//      choices[value]++;
//    else
//      choices[value] = 1;
  }
  ret['voteStats'] = choices;

  return ret;
}

function getSpecialUsersRecs(data, su)
{
  var ret = new Array();
  if (su != null && su != undefined) {
    for (var i = 0; i < data.length; ++i) {
      //if (isSpecialUser(su, data[i].get("userId")))
      if (isSpecialUser(su, data[i]['userId']))
        ret.push(data[i]);
    }
  }
  
  //keep only the most recent record of each user
  ret = getDiffUsersRecs(ret);
  
  return ret;
}

function getDiffUsersRecs(data)
{
  var ret = new Array();  //to know how many diff users are there
  for (var i = 0; i < data.length; ++i) {
    //userId = data[i].get("userId");
    userId = data[i]['userId'];
    //rows in data are descending in time, so skip it when already exists
    if (!userExists(ret, userId)) {
      ret.push(data[i]);
    }
  }
  return ret;
}

//TODO combine the 2 funcs below as a single utility func
function isSpecialUser(su, userId)
{
  for (var i = 0; i < su.length; ++i) {
    if (userId == su[i])
      return true;
  }
  return false;
}

function userExists(userArray, userId)
{
  for (var i = 0; i < userArray.length; ++i) {
    //var curUserId = userArray[i].get("userId");
    var curUserId = userArray[i]['userId'];
    if (curUserId != null && curUserId != undefined && curUserId == userId)
      return true;
  }
  return false;
}

function findMax(array) {
  var ret = -10000;
  for (var i = 0; i < array.length; ++i) {
     if (array[i] != undefined && array[i] != null && array[i] > ret)
      ret = array[i];
  }
  return ret;
}

function findMin(array) {
  var ret = 10000;
  for (var i = 0; i < array.length; ++i) {
     if (array[i] != undefined && array[i] != null && array[i] < ret)
      ret = array[i];
  }
  return ret;
}
